
require "string"
require "math"

----------------------------------------------------------------------------------------------------------------
-- utils

function string:split(sep)
        local sep, fields = sep or ":", {}
        local pattern = string.format("([^%s]+)", sep)
        self:gsub(pattern, function(c) fields[#fields+1] = c end)
        return fields
end

----------------------------------------------------------------------------------------------------------------
-- global variables

next_time_redo = 0
is_game_over = 0

is_player_in_water = false

----------------------------------------------------------------------------------------------------------------
-- load and save

function callSaveGame()
	saveGame()
	next_time_redo = getTime() + 1000
end

function callLoadGame()
	loadGame()
	is_game_over = 0
	is_player_in_water = false
	next_time_redo = getTime() + 1000
end

----------------------------------------------------------------------------------------------------------------

is_editing_flag = false
is_deleting_flag = false

current_flag_text = ''
current_flag_pos_x = 0
current_flag_pos_y = 0
current_flag_pos_z = 0
model_Flag = 0

Flag = {

	load = function()
	end,
	
	init = function()
		local x,y,z=getCurrentObjectPosition()
		local caption = getObjectLocalVariableString('init')
	end,

	manage = function()
		if getDistanceToPlayer() < 15 and not isEditorMode() and not is_editing_flag and not is_deleting_flag then
			current_flag_text = getObjectLocalVariableString('init')
			local x,y,z=getCurrentObjectPosition()
			current_flag_pos_x = x/10
			current_flag_pos_y = y/10
			current_flag_pos_z = z/10
		end
	end,

	draw = function()
		draw3DModel(model_Flag)
	end,
	
	uninit = function()
	end,

	unload = function()
	end
}

function loadFlags()

	local url = URL_SERVER..'/get_captions.php'
	print (url)
	local page = getPageFromWebServer(url, false)
	for index, value in ipairs(string.split(page, "\n")) do
		local line = string.split(value, '|')
		local x = line[1]
		local y = line[2]
		local z = line[3]
		local z = line[3]
		local type_content = line[4]
		local caption = line[5]
		print(x..';'..y..';'..z..';'..type_content..';'..caption)
		if type_content == 'text' then
			createObject(x*10,y*10,z*10, 'Flag', false, caption)
		end
	end

end

----------------------------------------------------------------------------------------------------------------

Game = {

	sound_main_music = 0,
	model_delta_wing = 0,
	time_main_music = 0,
	compass_sprite = 0,
	compass_arrow_sprite = 0,
	commands_sprite = 0,
	edit_text = 0,
	empty_for_text_sprite = 0,

	--------------------------------------------------------

	init = function()
		setFogColor(180,200,255)
		sound_main_music = loadSound('sound/ocean.ogg')
		fall_in_water_sprite = load2DSprite('images/fall_in_water.png')
		compass_sprite = load2DSprite('images/compass.png')
		compass_arrow_sprite = load2DSprite('images/compass_arrow.png')
		model_delta_wing = load3DModel('models/obj_delta_wing/delta_wing.obj')
		empty_for_text_sprite = load2DSprite('images/empty_for_text.png')
		time_main_music = getTime()
		--initActiveObjects('Gun','MagicSphere','Inhabitant','Pipe','Path1','Path2','DormRoom','Market')
		initActiveObjects('Flag')
		--setPlayerPositionReal(5640, 320, 20)
		setPlayerPositionReal(INIT_X_POS, INIT_Y_POS, INIT_Z_POS)
		callSaveGame()
		--setPlayerMovementCar()
		commands_sprite = load2DSprite('images/commands.png')
		
		model_Flag = load3DModelAnimated(100, 
			'models/obj_flag/flag.obj',
			'models/obj_flag/flag2.obj',
			'models/obj_flag/flag3.obj',
			'models/obj_flag/flag4.obj',
			'models/obj_flag/flag5.obj',
			'models/obj_flag/flag6.obj')
		
		loadFlags()
	end,
	
	--------------------------------------------------------

	uninit = function()
		unload3DModel(model_Flag)
		unload2DSprite(fall_in_water_sprite)
		unload2DSprite(compass_sprite)
		unload2DSprite(compass_arrow_sprite)
		unload3DModel(model_delta_wing)
		if commands_sprite ~= 0 then
			unload2DSprite(commands_sprite)
		end
		unload2DSprite(empty_for_text_sprite)
	end,
	
	--------------------------------------------------------
	
	manage = function()
	
		if true then -- ambient water sound

			if getTime() > time_main_music then
				time_main_music = getTime() + 500000
					playSound(sound_main_music)
					setSoundVolume(sound_main_music, 0.0)
			end
		
			local xcam,ycam,zcam = getCameraPositionReal()
			if ycam > 300.0 and isPlayingSound(sound_main_music) then 
				setSoundVolume(sound_main_music, 0.0)
			elseif ycam < 0.0 then
				setSoundVolume(sound_main_music, 0.0)
			else
				setSoundVolume(sound_main_music, 1.0 - ycam/300.0)
				--setSoundVolume(sound_main_music, 0.0)
			end
		end
		
		if getKeyNoRepeat('a') and not is_editing_flag and not is_deleting_flag then
			setEditorMode(not isEditorMode())
		end
		
	end,
	
	--------------------------------------------------------
	
	draw3D = function()
	
		if not is_editing_flag and not is_deleting_flag and getKeyNoRepeat("ASCII:27") then --escape key
			startMainMenu()
		end
	
		if not isEditorMode() then
		
			x, y, z = getPlayerPositionReal()
			
			-- fall into the water
			if y < -0.0 and not is_player_in_water then
				--playSound(sound_splash)
				is_player_in_water = true
			end
			
			if not is_editing_flag and not is_deleting_flag then
				-- reload game
				if getKey('l') then
					callLoadGame()
				end
				
				-- fly
				if ALLOW_FLYING and getKey('r') then
					speedx, speedy, speedz = getPlayerSpeed()
					setPlayerSpeed(speedx, 1.0, speedz)
				end
				
				-- start removing flag
				if ALLOW_DELETING_FLAG and getKeyNoRepeat("ASCII:127") and current_flag_text ~= '' then -- delete key
					is_deleting_flag = true
					setPlayerMovementNone()
				end
				
				-- start adding flag
				if getKeyNoRepeat("ASCII:10") or getKeyNoRepeat("ASCII:13") and current_flag_text == '' then
					is_editing_flag = true
					local window_width, window_height = getWindowSize()
					edit_text = createEditText(window_width/2-250, window_height/2-10, 50, 
						"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 ./:?&")
					setPlayerMovementNone()
				end
			end
			
			if is_deleting_flag then
				if getKeyNoRepeat("ASCII:27") then -- cancel flag remove (escape key)
					is_deleting_flag = false
					setPlayerMovementWalk()
				elseif (getKeyNoRepeat("ASCII:10") or getKeyNoRepeat("ASCII:13")) then
					local url = URL_SERVER..'/del_caption.php?x='..
						current_flag_pos_x..'&y='..current_flag_pos_y..'&z='..current_flag_pos_z
					print(url)
					page = getPageFromWebServer(url, false)
					if page == 'OK' then
						scheduleDraw2DSprite('images/empty_for_text.png',1000)
						scheduleDrawTextCenter('Ok.', 1000)
						removeObject(current_flag_pos_x*10,current_flag_pos_y*10,current_flag_pos_z*10)
					else
						scheduleDraw2DSprite('images/empty_for_text.png',3000)
						scheduleDrawTextCenter('Could not remove this flag\nfor some reason.', 3000)
					end
					is_deleting_flag = false
					setPlayerMovementWalk()
				end
			end
			
			if is_editing_flag then
				if getKeyNoRepeat("ASCII:27") then -- cancel flag add (escape key)
					is_editing_flag = false
					removeEditTextAndGetText()
					setPlayerMovementWalk()
				elseif getKeyNoRepeat("ASCII:10") or getKeyNoRepeat("ASCII:13") then -- confirm flag add (enter key)
					local caption = removeEditTextAndGetText()
					if not (caption == '') then
						local caption2 = string.gsub(caption, ' ', "%%20") -- replace space by '%20' for url
						caption2 = string.gsub(caption2, '%!', '%%21')
						caption2 = string.gsub(caption2, '%&', '%%26')
						caption2 = string.gsub(caption2, '%/', '%%2F')
						caption2 = string.gsub(caption2, '%?', '%%3F')
						local x,y,z = getPlayerPositionReal()
						local x=math.floor(x/10)
						local y=math.floor(y/10)
						local z=math.floor(z/10)
						local url = URL_SERVER..'/put_caption.php?x='..x..'&y='..y..'&z='..z..'&type=text&caption='..caption2
						print(url)
						page = getPageFromWebServer(url, false)
						if not (page == 'OK') then
							scheduleDraw2DSprite('images/empty_for_text.png',3000)
							scheduleDrawTextCenter('Sorry, the caption could not be\nadded for some reason.', 3000)
						else
							createObject(x*10,y*10,z*10, 'Flag', false, caption)
						end
					end
					setPlayerMovementWalk()
					is_editing_flag = false
				end
			end
			
			local is_left_click, is_middle_click, is_right_click, is_wheel_up, is_wheel_down, pos_x, pos_y = getMouseInfos()
			if is_left_click then
				speedx, speedy, speedz = getPlayerSpeed()
				if speedy < -0.0 then
					setCurrentPositionReal(getPlayerPositionReal())
					local angle_x, angle_y, dist = getCameraAngleAndDistance()
					rotatePosition(0,angle_y,0)
					draw3DModel(model_delta_wing)
					if speedx > 0.0 then
						setPlayerSpeed(6.0, -0.2, speedz)
					else
						setPlayerSpeed(speedx, -0.2, speedz)
					end
				end
			end
		end
	end,
	
	--------------------------------------------------------
	
	draw2D = function()
	
		local window_width, window_height = getWindowSize()

		if not isEditorMode() then
				
			if is_player_in_water then
				setPlayerMovementNone()
				setColor(255,255,255,255)
				if not isEditorMode() then
					draw2DSprite(fall_in_water_sprite, window_width/2 - 128, window_height/2 - 128)
				end
				local px,py,pz = getPlayerPositionReal()
				setPlayerPositionReal(px, -10.0, pz)
			end
			
			local angle_x, angle_y, camera_distance_to_player = getCameraAngleAndDistance()
				
			if camera_distance_to_player == 0 then
				draw2DSprite(compass_sprite, window_width/2 - 128, window_height - 256)
				draw2DSprite(compass_arrow_sprite, window_width/2, window_height - 80, angle_y, 64,64)
			end
		
			if commands_sprite ~= 0 then
				draw2DSprite(commands_sprite,10,10)
				if getKey(' ') then
					unload2DSprite(commands_sprite)
					commands_sprite = 0
				end
			end
			
			if is_editing_flag then
				draw2DSprite(empty_for_text_sprite)
				drawText(
					'Please enter a caption for this flag,\n'..
					'and then press Enter.\n'..
					'Note: this will be available for all players,\n'..
					'so please take care about what you write.',
					window_width/2-250,window_height/2-100)
					--'Note: this can\'t be removed and\nwill be available for all players,\nso please take care about what you write.'
			elseif is_deleting_flag then
				draw2DSprite(empty_for_text_sprite)
				drawText(
					'Note: please consider when removing\n'..
					'a flag, the player who set it up might\n'..
					'be upset.\n'..
					'\n'..
					'Are you sure you want to remove this\n'..
					'flag? Press Enter to confirm\n'..
					'or Esc to cancel.',
					window_width/2-250,window_height/2-100)
			elseif current_flag_text ~= '' then
				draw2DSprite(empty_for_text_sprite)
				drawText(current_flag_text)
			end
		end
		current_flag_text = ''
	end
}
